/* -----------------------------------------------------------------------------
 * Copyright (c) 2019 - 2021 ARM Ltd.
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from
 * the use of this software. Permission is granted to anyone to use this
 * software for any purpose, including commercial applications, and to alter
 * it and redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software in
 *    a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 *
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 3. This notice may not be removed or altered from any source distribution.
 *
 *
 * $Date:        01. October 2021
 * $Revision:    V1.1.0
 *
 * Project:      Flash Programming Functions for S32K3
 * --------------------------------------------------------------------------- */

/* History:
 *  Version 1.1.0
 *    Added S32K3x2 support
 *  Version 1.0.0
 *    Initial release
 */

#include <stdint.h>
#include "..\FlashOS.h"        /* FlashOS Structures */

#include "Fls_Api.h"
#include "Fls_Type.h"
#include "StdRegMacros.h"
#include "Fls_Registers.h"


#if defined (S32K3_CODE)
    #define START_ADDRESS       FLS_CODE_BASE_ADDRESS
    #define START_SECTOR        FLS_CODE_ARRAY_0_BLOCK_0_S000
	#ifdef S32K3X4
    #define END_SECTOR          FLS_CODE_ARRAY_0_BLOCK_3_S495        /* 0x007DE000 */
	#else
    #define END_SECTOR          FLS_CODE_ARRAY_0_BLOCK_1_S239        /* 0x005DE000 */
	#endif
    #define NUM_SECTORS         (END_SECTOR - START_SECTOR + 1U)
  /* Do not use sBAF managed sectores:
      sectors FLS_CODE_ARRAY_0_BLOCK_3_S496 ? ...  are used by SBAF
   */
#elif defined (S32K3_DATA)
    #define START_ADDRESS       FLS_DATA_BASE_ADDRESS
    #define START_SECTOR        FLS_DATA_ARRAY_0_BLOCK_4_S000
    #define END_SECTOR          FLS_DATA_ARRAY_0_BLOCK_4_S015
    #define NUM_SECTORS         (END_SECTOR - START_SECTOR + 1U)
#endif


#define XRDC_BASEADDR                 ((uint32_t)(0x40278000U))
#define XRDC_HWCFG1_ADDR32            ((uint32_t)(XRDC_BASEADDR + (uint32_t)0xF4U))  /* XRDC Hardware Configuration 1 (HWCFG1) */

int __disable_irq(void);

static uint32        Sectors[NUM_SECTORS];
static FLASH_CONFIG  FlsConfigSet;
static uint8         FlsDomainIdValue;
extern uint8 u8SecureCounter;

/*
 *  Initialize Flash Programming Functions
 *    Parameter:      adr:  Device Base Address
 *                    clk:  Clock Frequency (Hz)
 *                    fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */
int Init (unsigned long adr, unsigned long clk, unsigned long fnc) {
  int ret;
  uint32_t i;

  (void) adr;
  (void) clk;
  (void) fnc;

  __disable_irq();


  // Initialize flash configuration structure
  for (i = 0U; i < NUM_SECTORS; i++) {
    Sectors[i] = START_SECTOR + i;
  }

  FlsConfigSet.Fls_bEnableTimeOut = STD_ON;
  FlsConfigSet.Fls_u32ValueWaitDoneBitOrDomainIDsTimeOut = 1000020U;
  FlsConfigSet.Fls_pAllSectors = Sectors;
  FlsConfigSet.Fls_u32NumberOfconfiguredSectors = NUM_SECTORS;
  FlsConfigSet.Fls_InterfaceAccess = FLS_MAIN_INTERFACE;

  FlsDomainIdValue = (uint8)(REG_READ32(XRDC_HWCFG1_ADDR32) & 0x0000000F);

  // Call init function
  u8SecureCounter = 0U;
  ret = FlashInit(&FlsConfigSet);

  if (ret == FLS_JOB_OK)
  {
    return (0);                                               /* Finished without Errors */
  }
  else
  {
    return (1);                                               /* Finished with Errors */
  }
}


/*
 *  De-Initialize Flash Programming Functions
 *    Parameter:      fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */

int UnInit (unsigned long fnc) {

  (void) fnc;

  return (0);                                                 /* Finished without Errors */
}


/*
 *  Erase complete Flash Memory
 *    Return Value:   0 - OK,  1 - Failed
 */
int EraseChip (void) {
  int ret;

  ret = FlashErase(0U, NUM_SECTORS*FLS_SECTOR_SIZE, STD_OFF, FlsDomainIdValue);  /* erase only sector 0..487 */
  if (ret == FLS_JOB_OK)
  {
    return (0);                                               /* Finished without Errors */
  }
  else
  {
    return (1);                                               /* Finished with Errors */
  }
}


/*
 *  Erase Sector in Flash Memory
 *    Parameter:      adr:  Sector Address
 *    Return Value:   0 - OK,  1 - Failed
 */
int EraseSector (unsigned long adr) {
  int ret;

  ret = FlashErase(adr - START_ADDRESS, FLS_SECTOR_SIZE, STD_OFF, FlsDomainIdValue);
  if (ret == FLS_JOB_OK)
  {
    return (0);                                               /* Finished without Errors */
  }
  else
  {
    return (1);                                               /* Finished with Errors */
  }
}


/*
 *  Program Page in Flash Memory
 *    Parameter:      adr:  Page Start Address
 *                    sz:   Page Size
 *                    buf:  Page Data
 *    Return Value:   0 - OK,  1 - Failed
 */
int ProgramPage (unsigned long adr, unsigned long sz, unsigned char *buf) {
  int ret;

  if ((sz % FLS_WRITE_DOUBLE_WORD) != 0U ) {
    sz = (sz & ~7U) + 8U;
  }

  ret = FlashProgram(adr - START_ADDRESS, buf, sz, STD_OFF, STD_OFF, FlsDomainIdValue);
  if (ret == FLS_JOB_OK)
  {
    return (0);                                               /* Finished without Errors */
  }
  else
  {
    return (1);                                               /* Finished with Errors */
  }
}
