/*****************************************************************************
*            (c) Copyright Freescale Semiconductor, 2011-2018                *
*                         All Rights Reserved                                *
******************************************************************************

******************************************************************************
*                                                                            *
*               Standard Software Driver for C40                             *
*                                                                            *
* FILE NAME     : FlashArrayIntegrityCheck.c                                 *
* FUNCTION      : FlashArrayIntegrityCheck                                   *
* DESCRIPTION   : This function will will check the array integrity          *
*                 of the Flash
* PARAMETERS    :
* RETURN VALUES : 
******************************************************************************/
/**
* @page misra_violations MISRA-C:2012 violations
*
* @section FlashArrayIntegrityCheck_c_REF_1
* Violates MISRA 2012 Advisory Rule 15.5, A function should have a single point of exit at the end
* This violation is not fixed since if hardware/configuration errors are detected it should return from the function
*
* @section FlashArrayIntegrityCheck_c_REF_2
* Violates MISRA 2012 Advisory Rule 11.4, A conversion should not be performed between a pointer to object and an integer type.
* The cast is used to access memory mapped registers.
*
* @section FlashArrayIntegrityCheck_c_REF_3
* Violates MISRA 2012 Required Rule 11.6, A cast shall not be performed between pointer to void and an arithmetic type.
* This violation is due to casting unsigned long to pointer and access these addresses for updating  contents in that address.
*
* @section FlashArrayIntegrityCheck_c_REF_4
* Violates MISRA 2012 Required Rule 10.8, The value of a composite expression shall no be cast to a different essential type category or a wider essential type.
* The cast is used to access memory mapped registers.
*
* @section FlashArrayIntegrityCheck_c_REF_5
* Violates MISRA 2012 Advisory Rule 8.7, Functions and objects should not be defined with external linkage if
* they are referenced in only one translation unit.
* These functions represent the API of the driver. External linkage is needed to "export" the API.
*
*/
#include "Fls_Registers.h"
#include "Fls_Api.h"

/* @violates @ref FlashArrayIntegrityCheck_c_REF_5 Functions and objects should not be defined with external linkage if
* they are referenced in only one translation unit. */
Fls_CheckStatusType FlashArrayIntegrityCheck( Fls_FlashBlocksType eSelectBlock,
                                              const uint8 *pMisrRegDesiredValues, 
                                              const uint8 *pMisrExpectValues, 
                                              Fls_ArrayIntegritySequenceType eAddressSequence,
                                              Fls_FlashBreakPointsType eBreakPoints
                                            )
{
    Fls_CheckStatusType eReturnCode = FLS_JOB_FAILED;     /* return code */
    uint32 u32TempMisrRegDesiredValues = 0U;
    uint32 u32TempMisrExpectValues = 0U;
    uint8 u8Counter = 0U;
    uint32 u32ValueTimeOut = 0U;
    
    /* A secure counter will help to determine the attack if any hacker trying to skipping some instructions */
    u8SecureCounter = 1U;
    u8SecureCounter++;
    /* check if it is wrong input block */
    if(
        (FLS_BLOCK_0 == eSelectBlock) || (FLS_BLOCK_1 == eSelectBlock) || (FLS_BLOCK_2 == eSelectBlock) ||
#ifdef S32K3X4
        (FLS_BLOCK_3 == eSelectBlock) || (FLS_BLOCK_4 == eSelectBlock) ||
#endif
        (FLS_UTEST == eSelectBlock)
      )
    {
        /* Input parameter was correct */
    }
    else
    {
        /* Wrong input parameters*/
        /* @violates @ref FlashArrayIntegrityCheck_c_REF_1 A function should have a single point of exit at the end */
        return FLS_INPUT_PARAM_FAILED;
    }
    /* if null pointer */
    if((pMisrRegDesiredValues == NULL_PTR) || (pMisrExpectValues == NULL_PTR))
    {
        /* Wrong input parameters*/
        /* @violates @ref FlashArrayIntegrityCheck_c_REF_1 A function should have a single point of exit at the end */
        return FLS_INPUT_PARAM_FAILED;
    }
    /* Checking if hackers tried to skip some instructions or not*/
    if(2U == u8SecureCounter)
    {
        u8SecureCounter ++;
    }
    else
    {
        /* There were hackers who skipped some instructions */
        /* @violates @ref FlashArrayIntegrityCheck_c_REF_1 A function should have a single point of exit at the end */
        return FLS_JOB_FAILED;
    }
    /* There is an in-progress UTest operation */
    /*
    * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
    * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
    */
    if(0U != REG_BIT_GET32(FLASH_UT0_ADDR32, FLASH_UT0_AIE_U32))
    {
        /*Reset timeout value*/
        u32ValueTimeOut = u32ValueWaitDoneBitOrDomainIDsTimeOut;
        do
        {
            /*if UT0[NAIBP] bit was enabled */
            /*
            * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
            * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
            */
            if(0U != REG_BIT_GET32(FLASH_UT0_ADDR32, FLASH_UT0_NAIBP_U32))
            {
                /* cleared MCRS[SBC] bit write 1 to clear)*/
                /*
                * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
                * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
                */
                REG_BIT_SET32(FLASH_MCRS_ADDR32, FLASH_MCRS_SBC_W1C);
                /* cleared MCRS[EER] bit write 1 to clear)*/
                /*
                * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
                * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
                */
                REG_BIT_SET32(FLASH_MCRS_ADDR32, FLASH_MCRS_EER_W1C);
                /*
                * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
                * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
                */
                REG_BIT_CLEAR32(FLASH_UT0_ADDR32, FLASH_UT0_NAIBP_U32);
                /* Break a single bit correction*/ 
            }
            /* When enable the Timeout*/
            if((boolean)STD_ON == bEnableTimeOut)
            {
                /*Decrease the Timeout value*/
                u32ValueTimeOut--;
                if(0U == u32ValueTimeOut)
                {
                    /*Errors Timeout because wait for the Done bit long time*/
                    /* @violates @ref FlashArrayIntegrityCheck_c_REF_1 A function should have a single point of exit at the end */
                    return FLS_TIMEOUT_FAILED;
                }
            }
            else
            {
                /*Waiting for Done bit */
            }
        /*
        * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
        * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
        */    
        }while(0U == REG_BIT_GET32(FLASH_UT0_ADDR32, FLASH_UT0_AID_U32));
    }
     /* Checking if hackers tried to skip some instructions or not*/
    if(3U == u8SecureCounter)
    {
        u8SecureCounter ++;
    }
    else
    {
        /* There were hackers who skipped some instructions */
        /* @violates @ref FlashArrayIntegrityCheck_c_REF_1 A function should have a single point of exit at the end */
        return FLS_JOB_FAILED;
    }
    /* Enable UTest mode*/
    /* The method to set this bit is to provide a password, and if the password matches */
    /*
    * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
    * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
    */
    REG_WRITE32(FLASH_UT0_ADDR32, C55_USER_TEST_PASSWORD);
    /*
    * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
    * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
    */
    if(0U != REG_BIT_GET32(FLASH_UT0_ADDR32, FLASH_UT0_UTE_U32))
    {
        /* Enable UTest mode successfully */
        eReturnCode = FLS_JOB_OK;
    }
    else
    {
        /* Did not enable the Utest mode yet*/
        /* @violates @ref FlashArrayIntegrityCheck_c_REF_1 A function should have a single point of exit at the end */
        return FLS_JOB_FAILED;
    }
     /* Checking if hackers tried to skip some instructions or not*/
    if(4U == u8SecureCounter)
    {
        u8SecureCounter ++;
    }
    else
    {
        /* There were hackers who skipped some instructions */
        /* @violates @ref FlashArrayIntegrityCheck_c_REF_1 A function should have a single point of exit at the end */
        return FLS_JOB_FAILED;
    }
    /* Select the block to receive the array integrity check by doing an interlock write to that block */
    /* 
    * The block selected for array integrity check do not need to be unlocked.
    * Write to the sector or block address to be erased using logical address registers located in the Platform Flash Controller
    */
    /*
    * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
    * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
    */
    REG_WRITE32(PFLASH_PFCPGM_PEADR_L_ADDR32, (uint32)eSelectBlock);
    /* One and only one ADATA register must also be written. This is referred to as an erase interlock write.*/
    /*
    * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
    * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
    */
    REG_WRITE32(FLASH_DATAx_ADDR32, (uint32)0xFFFFFFFFU);
    /*If desired, set the UT0[AIS] bit to 1 for sequential addressing only*/
    if ( FLS_PROPRIETARY_SEQENCE == eAddressSequence)
    {
        /* Array integrity sequence is proprietary sequence */
        /*
        * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
        * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
        */
        REG_BIT_CLEAR32(FLASH_UT0_ADDR32, FLASH_UT0_AIS_U32);
    }
    else if ( FLS_SEQUENTIAL == eAddressSequence)
    {
        /* Array integrity sequence is sequential */
        /*
        * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
        * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
        */
        REG_WRITE32(FLASH_UT0_ADDR32, FLASH_UT0_AIS_U32);
    }
    else
    {
        /* Wrong input parameters*/
        /* @violates @ref FlashArrayIntegrityCheck_c_REF_1 A function should have a single point of exit at the end */
        return FLS_INPUT_PARAM_FAILED;
    }
     /* Checking if hackers tried to skip some instructions or not*/
    if(5U == u8SecureCounter)
    {
        u8SecureCounter ++;
    }
    else
    {
        /* There were hackers who skipped some instructions */
        /* @violates @ref FlashArrayIntegrityCheck_c_REF_1 A function should have a single point of exit at the end */
        return FLS_JOB_FAILED;
    }
    /*Seed the MISR registers (UM0 - UM8) with desired values*/
    /*
    * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
    * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
    */
    u32TempMisrRegDesiredValues = (uint32)pMisrRegDesiredValues;
    for (u8Counter = 0U; u8Counter < 9U; u8Counter++)
    {
        /*
        * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
        * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
        * @violates @ref FlashArrayIntegrityCheck_c_REF_4 The value of a composite expression shall no be cast to a different essential type category or a wider essential type
        */
        REG_WRITE32(FLASH_UMx_ADDR32 + (uint32)(u8Counter*4U) , *(uint32 *)u32TempMisrRegDesiredValues);
        u32TempMisrRegDesiredValues += 4U;
    }
     /* Checking if hackers tried to skip some instructions or not*/
    if(6U == u8SecureCounter)
    {
        u8SecureCounter ++;
    }
    else
    {
        /* There were hackers who skipped some instructions */
        /* @violates @ref FlashArrayIntegrityCheck_c_REF_1 A function should have a single point of exit at the end */
        return FLS_JOB_FAILED;
    }
    /* Seed the MISR registers UM9 with desired values */
    /*
    * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
    * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
    */
     REG_WRITE32(FLASH_UM9_ADDR32, *(uint32 *)u32TempMisrRegDesiredValues);
    /* If breakpoints are desired, set UT0[AIBPE] to 1, and ensure that MCRS[EER] and MCRS[SBC] are cleared. If it is desired to break on single-bit correction, ensure that UT0[SBCE] is set */ 
    if(FLS_NO_BREAKPOINTS == eBreakPoints) 
    {
        /* Array Integrity breakpoints are not enabled */
        /*
        * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
        * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
        */
        REG_BIT_CLEAR32(FLASH_UT0_ADDR32, FLASH_UT0_AIBPE_U32);
    }
    else if(FLS_BREAKPOINTS_ON_DBD == eBreakPoints)
    {
        /* Ensure that MCRS[EER] and MCRS[SBC] are cleared (write 1 to clear)*/
        /*
        * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
        * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
        */
        REG_BIT_SET32(FLASH_MCRS_ADDR32, FLASH_MCRS_EER_W1C);
        /*
        * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
        * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
        */
        REG_BIT_SET32(FLASH_MCRS_ADDR32, FLASH_MCRS_SBC_W1C);
        /*Array Integrity breakpoints are enabled during Array Integrity Checks*/
        /*
        * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
        * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
        */
        REG_BIT_SET32(FLASH_UT0_ADDR32, FLASH_UT0_AIBPE_U32);
        /*Single Bit Correction observation is disabled.*/
        /*
        * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
        * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
        */
        REG_BIT_CLEAR32(FLASH_UT0_ADDR32, FLASH_UT0_SBCE_U32);
    }
    else if(FLS_BREAKPOINTS_ON_DBD_SBC == eBreakPoints)
    {
        /* Ensure that MCRS[EER] and MCRS[SBC] are cleared (write 1 to clear)*/
        /*
        * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
        * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
        */
        REG_BIT_SET32(FLASH_MCRS_ADDR32, FLASH_MCRS_EER_W1C);
        /*
        * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
        * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
        */
        REG_BIT_SET32(FLASH_MCRS_ADDR32, FLASH_MCRS_SBC_W1C);
        /*Array Integrity breakpoints are enabled during Array Integrity Checks*/
        /*
        * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
        * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
        */
        REG_BIT_SET32(FLASH_UT0_ADDR32, FLASH_UT0_AIBPE_U32);
        /*Single Bit Correction observation is enabled.*/
        /*
        * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
        * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
        */
        REG_BIT_SET32(FLASH_UT0_ADDR32, FLASH_UT0_SBCE_U32);
    }
    else
    {
        /* Wrong input parameters*/
        /* @violates @ref FlashArrayIntegrityCheck_c_REF_1 A function should have a single point of exit at the end */
        return FLS_INPUT_PARAM_FAILED;
    }
     /* Checking if hackers tried to skip some instructions or not*/
    if(7U == u8SecureCounter)
    {
        u8SecureCounter ++;
    }
    else
    {
        /* There were hackers who skipped some instructions */
        /* @violates @ref FlashArrayIntegrityCheck_c_REF_1 A function should have a single point of exit at the end */
        return FLS_JOB_FAILED;
    }
    /* Array integrity sequence is sequential */
    /*
    * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
    * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
    */
    REG_BIT_SET32(FLASH_UT0_ADDR32, FLASH_UT0_AIE_U32);
    /* Wait until the UT0[AID] bit goes high*/
    /*Reset timeout value*/
    u32ValueTimeOut = u32ValueWaitDoneBitOrDomainIDsTimeOut;
    do
    {
        /*If breakpoints were enabled, check UT0[NAIBP]*/
        if(FLS_NO_BREAKPOINTS != eBreakPoints)
        {
            /*if UT0[NAIBP] bit was enabled */
            /*
            * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
            * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
            */
            if(0U != REG_BIT_GET32(FLASH_UT0_ADDR32, FLASH_UT0_NAIBP_U32))
            {
                /*
                * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
                * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
                */
                if(0U != REG_BIT_GET32(FLASH_MCRS_ADDR32, FLASH_MCRS_SBC_W1C))
                {
                    /* cleared MCRS[SBC] bit write 1 to clear)*/
                    /*
                    * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
                    * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
                    */
                    REG_BIT_SET32(FLASH_MCRS_ADDR32, FLASH_MCRS_SBC_W1C);
                    /*
                    * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
                    * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
                    */
                    REG_BIT_CLEAR32(FLASH_UT0_ADDR32, FLASH_UT0_NAIBP_U32);
                    /* Break a single bit correction*/
                    /* @violates @ref FlashArrayIntegrityCheck_c_REF_1 A function should have a single point of exit at the end */
                    return FLS_USER_TEST_BREAK_SBC;
                }
                /*
                * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
                * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
                */
                else if(0U != REG_BIT_GET32(FLASH_MCRS_ADDR32, FLASH_MCRS_EER_W1C))
                {
                    /* cleared MCRS[EER] bit write 1 to clear)*/
                    /*
                    * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
                    * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
                    */
                    REG_BIT_SET32(FLASH_MCRS_ADDR32, FLASH_MCRS_EER_W1C);
                    /*
                    * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
                    * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
                    */
                    REG_BIT_CLEAR32(FLASH_UT0_ADDR32, FLASH_UT0_NAIBP_U32);
                    /* Break double bit detection */
                    /* @violates @ref FlashArrayIntegrityCheck_c_REF_1 A function should have a single point of exit at the end */
                    return FLS_USER_TEST_BREAK_DBD;
                }
                else
                {
                    /*
                    * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
                    * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
                    */
                    REG_BIT_CLEAR32(FLASH_UT0_ADDR32, FLASH_UT0_NAIBP_U32);
                    /* Had errors when NAIBP was set but not the Break Point*/
                    /* @violates @ref FlashArrayIntegrityCheck_c_REF_1 A function should have a single point of exit at the end */
                    return FLS_JOB_FAILED;
                }
            }
        }
        /* When enable the Timeout*/
        if((boolean)STD_ON == bEnableTimeOut)
        {
            /*Decrease the Timeout value*/
            u32ValueTimeOut--;
            if(0U == u32ValueTimeOut)
            {
                /*Errors Timeout because wait for the Done bit long time*/
                /* @violates @ref FlashArrayIntegrityCheck_c_REF_1 A function should have a single point of exit at the end */
                return FLS_TIMEOUT_FAILED;
            }
        }
        else
        {
            /*Waiting for Done bit */
        }
    /*
    * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
    * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
    */    
    }while(0U == REG_BIT_GET32(FLASH_UT0_ADDR32, FLASH_UT0_AID_U32));
    /* Checking if hackers tried to skip some instructions or not*/
    if(8U == u8SecureCounter)
    {
        u8SecureCounter ++;
    }
    else
    {
        /* There were hackers who skipped some instructions */
        /* @violates @ref FlashArrayIntegrityCheck_c_REF_1 A function should have a single point of exit at the end */
        return FLS_JOB_FAILED;
    }
    /* Read values in the MISR registers (UM0 - UM8) to ensure correct signature */
    /*
    * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
    * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
    */
    u32TempMisrExpectValues = (uint32)pMisrExpectValues;
    for (u8Counter = 0U; u8Counter < 9U; u8Counter++)
    {
        /*
        * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
        * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
        * @violates @ref FlashArrayIntegrityCheck_c_REF_4 The value of a composite expression shall no be cast to a different essential type category or a wider essential type
        */
        if(*(uint32 *)u32TempMisrExpectValues != REG_READ32(FLASH_UMx_ADDR32 + (uint32)(u8Counter*4U)))
        {
            /* @violates @ref FlashArrayIntegrityCheck_c_REF_1 A function should have a single point of exit at the end */
            return FLS_JOB_FAILED;
        }
        u32TempMisrExpectValues += 4U;
    }
     /* Checking if hackers tried to skip some instructions or not*/
    if(9U == u8SecureCounter)
    {
        u8SecureCounter ++;
    }
    else
    {
        /* There were hackers who skipped some instructions */
        /* @violates @ref FlashArrayIntegrityCheck_c_REF_1 A function should have a single point of exit at the end */
        return FLS_JOB_FAILED;
    }
    /* Read one bit left in the UM9[MISR0] register to ensure correct signature */
    /*
    * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
    * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
    */
    if((*(uint32 *)u32TempMisrExpectValues & 0x1U) != (REG_READ32(FLASH_UM9_ADDR32) & 0x1U))
    {
        eReturnCode = FLS_JOB_FAILED;
    }
    else
    {
        eReturnCode = FLS_JOB_OK;
    }
    /* Write a logic 0 to the UT0[AIE] bit */  
    /*
    * @violates @ref FlashArrayIntegrityCheck_c_REF_2 A conversion should not be performed between a pointer to object and an integer type
    * @violates @ref FlashArrayIntegrityCheck_c_REF_3 A cast shall not be performed between pointer to void and an arithmetic type
    */
    REG_BIT_CLEAR32(FLASH_UT0_ADDR32, FLASH_UT0_AIE_U32);
    
    /* Checking if hackers tried to skip some instructions or not*/
    if(FLS_WELL_KNOWN_SECURE_COUNTER != u8SecureCounter)
    {
        /* There were hackers who skipped some instructions */
        eReturnCode = FLS_JOB_FAILED;
    }
    
    return  eReturnCode;
}  

